@@ -29,13 +29,16 @@
 #include "nautilus-application.h"
 
 
-#include "file-manager/fm-bonobo-provider.h"
 #include "file-manager/fm-ditem-page.h"
 #include "file-manager/fm-desktop-icon-view.h"
 #include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-list-view.h"
-#include "file-manager/fm-search-list-view.h"
 #include "file-manager/fm-tree-view.h"
+#include "nautilus-information-panel.h"
+#include "nautilus-history-sidebar.h"
+#include "nautilus-notes-viewer.h"
+#include "nautilus-emblem-sidebar.h"
+#include "nautilus-image-properties-page.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -76,7 +79,6 @@
 #include <libnautilus-private/nautilus-metafile-factory.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-sound.h>
-#include <libnautilus-private/nautilus-bonobo-extensions.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-private/nautilus-desktop-link-monitor.h>
 #include <libnautilus-private/nautilus-directory-private.h>
@@ -121,28 +123,13 @@ create_object (PortableServer_Servant servant,
 	       CORBA_Environment *ev)
 {
 	BonoboObject *object;
-	FMDirectoryView *directory_view;
 	NautilusApplication *application;
 
-	if (strcmp (iid, NAUTILUS_ICON_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_icon_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, NAUTILUS_DESKTOP_ICON_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_desktop_icon_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, NAUTILUS_LIST_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_list_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, SEARCH_LIST_VIEW_IID) == 0) {
-		directory_view = FM_DIRECTORY_VIEW (g_object_new (fm_search_list_view_get_type (), NULL));
-		object = BONOBO_OBJECT (fm_directory_view_get_nautilus_view (directory_view));
-	} else if (strcmp (iid, SHELL_IID) == 0) {
+	if (strcmp (iid, SHELL_IID) == 0) {
 		application = NAUTILUS_APPLICATION (bonobo_object_from_servant (servant));
 		object = BONOBO_OBJECT (nautilus_shell_new (application));
 	} else if (strcmp (iid, METAFILE_FACTORY_IID) == 0) {
 		object = BONOBO_OBJECT (nautilus_metafile_factory_get_instance ());
-	} else if (strcmp (iid, TREE_VIEW_IID) == 0) {
-		object = BONOBO_OBJECT (g_object_new (fm_tree_view_get_type (), NULL));
 	} else {
 		object = CORBA_OBJECT_NIL;
 	}
@@ -162,12 +149,6 @@ nautilus_application_get_spatial_window_list (void)
 	return nautilus_application_spatial_window_list;
 }
 
-static CORBA_Object
-create_object_shortcut (const char *iid,
-			gpointer callback_data)
-{
-	return create_object (BONOBO_OBJREF (callback_data), iid, NULL);
-}
 
 static void
 nautilus_application_instance_init (NautilusApplication *application)
@@ -187,11 +168,20 @@ nautilus_application_instance_init (NautilusApplication *application)
 	g_signal_connect_object (gnome_vfs_get_volume_monitor (), "volume_mounted",
 				 G_CALLBACK (volume_mounted_callback), application, 0);
 
-	nautilus_bonobo_register_activation_shortcut (NAUTILUS_ICON_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (NAUTILUS_DESKTOP_ICON_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (NAUTILUS_LIST_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (SEARCH_LIST_VIEW_IID, create_object_shortcut, application);
-	nautilus_bonobo_register_activation_shortcut (TREE_VIEW_IID, create_object_shortcut, application);
+	/* register views */
+	fm_icon_view_register ();
+	fm_desktop_icon_view_register ();
+	fm_list_view_register ();
+
+	/* register sidebars */
+	nautilus_information_panel_register ();
+	fm_tree_view_register ();
+	nautilus_history_sidebar_register ();
+	nautilus_notes_viewer_register (); /* also property page */
+	nautilus_emblem_sidebar_register ();
+
+	/* register property pages */
+	nautilus_image_properties_page_register ();
 }
 
 NautilusApplication *
@@ -215,15 +205,9 @@ nautilus_application_destroy (BonoboObject *object)
 
 	application = NAUTILUS_APPLICATION (object);
 
-	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_ICON_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_DESKTOP_ICON_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (NAUTILUS_LIST_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (SEARCH_LIST_VIEW_IID);
-	nautilus_bonobo_unregister_activation_shortcut (TREE_VIEW_IID);
-	
 	nautilus_bookmarks_exiting ();
 	
-	bonobo_object_unref (application->undo_manager);
+	g_object_unref (application->undo_manager);
 
 	EEL_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
 }
@@ -373,7 +357,6 @@ finish_startup (NautilusApplication *application)
 	/* initialize nautilus modules */
 	nautilus_module_init ();
 
-	nautilus_module_add_type (FM_TYPE_BONOBO_PROVIDER);
 	nautilus_module_add_type (FM_TYPE_DITEM_PAGE);
 	
 	/* initialize the sound machinery */
@@ -444,6 +427,23 @@ initialize_kde_trash_hack (void)
 	g_free (desktop_dir);
 }
 
+
+static Bonobo_RegistrationResult
+nautilus_bonobo_activation_register_for_display (const char    *iid,
+						 Bonobo_Unknown ref)
+{
+	const char *display_name;
+	GSList *reg_env ;
+	Bonobo_RegistrationResult result;
+	
+	display_name = gdk_display_get_name (gdk_display_get_default());
+	reg_env = bonobo_activation_registration_env_set (NULL,
+							  "DISPLAY", display_name);
+	result = bonobo_activation_register_active_server (iid, ref, reg_env);
+	bonobo_activation_registration_env_free (reg_env);
+	return result;
+}
+
 void
 nautilus_application_startup (NautilusApplication *application,
 			      gboolean kill_shell,
@@ -822,6 +822,9 @@ find_parent_spatial_window (NautilusSpatialWindow *window)
 	char *desktop_directory;
 
 	location = nautilus_window_get_location (NAUTILUS_WINDOW (window));
+	if (location == NULL) {
+		return NULL;
+	}
 	file = nautilus_file_get (location);
 	g_free (location);
 
@@ -931,9 +934,10 @@ create_window (NautilusApplication *application,
 	
 	window = NAUTILUS_WINDOW (gtk_widget_new (window_type,
 						  "app", application,
-						  "app_id", "nautilus",
 						  "screen", screen,
 						  NULL));
+	/* Must be called after construction finished */
+	nautilus_window_constructed (window);
 
 	g_signal_connect_data (window, "delete_event",
 			       G_CALLBACK (nautilus_window_delete_event_callback), NULL, NULL,
@@ -964,6 +968,20 @@ nautilus_application_present_spatial_window (NautilusApplication *application,
 					     NautilusWindow      *requesting_window,
 					     const char          *location,
 					     GdkScreen           *screen)
+{
+	return nautilus_application_present_spatial_window_with_selection (application,
+									   requesting_window,
+									   location,
+									   NULL,
+									   screen);
+}
+
+NautilusWindow *
+nautilus_application_present_spatial_window_with_selection (NautilusApplication *application,
+							    NautilusWindow      *requesting_window,
+							    const char          *location,
+							    GList		 *new_selection,
+							    GdkScreen           *screen)
 {
 	NautilusWindow *window;
 	GList *l;
@@ -977,13 +995,16 @@ nautilus_application_present_spatial_window (NautilusApplication *application,
                	     
 		existing_window = NAUTILUS_WINDOW (l->data);
 		existing_location = existing_window->details->pending_location;
-                
+		
 		if (existing_location == NULL) {
 			existing_location = existing_window->details->location;
 		}
 
 		if (eel_uris_match (existing_location, location)) {
 			gtk_window_present (GTK_WINDOW (existing_window));
+			if (new_selection) {
+				nautilus_view_set_selection (existing_window->content_view, new_selection);
+			}
 			return existing_window;
 		}
 	}
@@ -1016,7 +1037,7 @@ nautilus_application_present_spatial_window (NautilusApplication *application,
 	g_object_weak_ref (G_OBJECT (window), 
 			   spatial_window_destroyed_callback, NULL);
 	
-	nautilus_window_go_to (window, location);
+	nautilus_window_go_to_with_selection (window, location, new_selection);
 	
 	return window;
 }
@@ -1143,7 +1164,11 @@ volume_unmounted_callback (GnomeVFSVolumeMonitor *monitor,
 	/* Handle the windows in the close list. */
 	for (node = close_list; node != NULL; node = node->next) {
 		window = NAUTILUS_WINDOW (node->data);
-		nautilus_window_close (window);
+		if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
+			nautilus_window_close (window);
+		} else {
+			nautilus_window_go_home (window);
+		}
 	}
 		
 	g_list_free (close_list);
